/*---------------------------------------------------------------------\
|                          ____ _   __ __ ___                          |
|                         |__  / \ / / . \ . \                         |
|                           / / \ V /|  _/  _/                         |
|                          / /__ | | | | | |                           |
|                         /_____||_| |_| |_|                           |
|                                                                      |
\---------------------------------------------------------------------*/
/** \file	zypp/solver/Types.h
 *
 * Common types used in the Resolver public API and need to be passed
 * down to the details tree. Not for common use.
 */
#ifndef ZYPP_SOLVER_TYPES_H
#define ZYPP_SOLVER_TYPES_H

#include <zypp/ResPool.h>

#include <zypp/ResolverFocus.h>
#include <zypp/ResolverNamespace.h>

#include <zypp/ResolverProblem.h>

#endif // ZYPP_SOLVER_TYPES_H
