/*---------------------------------------------------------------------\
|                          ____ _   __ __ ___                          |
|                         |__  / \ / / . \ . \                         |
|                           / / \ V /|  _/  _/                         |
|                          / /__ | | | | | |                           |
|                         /_____||_| |_| |_|                           |
|                                                                      |
\---------------------------------------------------------------------*/
/** \file zypp-curl/proxyinfo/proxyinfos.h
 *
*/
#ifndef ZYPP_CURL_PROXYINFO_PROXYINFOS_H_INCLUDED
#define ZYPP_CURL_PROXYINFO_PROXYINFOS_H_INCLUDED

#include <string>
#include <list>

#include <zypp-curl/proxyinfo/ProxyInfoSysconfig>
#ifdef WITH_LIBPROXY_SUPPORT
#include <zypp-curl/proxyinfo/ProxyInfoLibproxy>
#endif

#endif // ZYPP_CURL_PROXYINFO_PROXYINFOS_H_INCLUDED
